@@ -1,5 +1,6 @@
 // Copyright (C) 2022 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include <QtHttpServer/qabstracthttpserver.h>
 
@@ -13,8 +14,10 @@
 #include <QtCore/qloggingcategory.h>
 #include <QtNetwork/qtcpserver.h>
 #include <QtNetwork/qtcpsocket.h>
+#if QT_CONFIG(localserver)
 #include <QtNetwork/qlocalserver.h>
 #include <QtNetwork/qlocalsocket.h>
+#endif
 
 #if QT_CONFIG(ssl)
 #include <QtNetwork/qsslserver.h>
@@ -28,7 +31,7 @@
 
 QT_BEGIN_NAMESPACE
 
-Q_LOGGING_CATEGORY(lcHttpServer, "qt.httpserver")
+Q_STATIC_LOGGING_CATEGORY(lcHttpServer, "qt.httpserver")
 
 /*!
     \internal
@@ -49,9 +52,9 @@ void QAbstractHttpServerPrivate::handleNewConnections()
         while (auto socket = qobject_cast<QSslSocket *>(sslServer->nextPendingConnection())) {
             if (socket->sslConfiguration().nextNegotiatedProtocol()
                             == QSslConfiguration::ALPNProtocolHTTP2) {
-                new QHttpServerHttp2ProtocolHandler(q, socket);
+                new QHttpServerHttp2ProtocolHandler(q, socket, &requestFilter);
             } else {
-                new QHttpServerHttp1ProtocolHandler(q, socket);
+                new QHttpServerHttp1ProtocolHandler(q, socket, &requestFilter);
             }
         }
         return;
@@ -62,7 +65,7 @@ void QAbstractHttpServerPrivate::handleNewConnections()
     Q_ASSERT(tcpServer);
 
     while (auto socket = tcpServer->nextPendingConnection())
-        new QHttpServerHttp1ProtocolHandler(q, socket);
+        new QHttpServerHttp1ProtocolHandler(q, socket, &requestFilter);
 }
 
 /*!
@@ -90,7 +93,7 @@ void QAbstractHttpServerPrivate::handleNewLocalConnections()
     Q_ASSERT(localServer);
 
     while (auto socket = localServer->nextPendingConnection())
-        new QHttpServerHttp1ProtocolHandler(q, socket);
+        new QHttpServerHttp1ProtocolHandler(q, socket, &requestFilter);
 }
 #endif
 
@@ -104,7 +107,7 @@ void QAbstractHttpServerPrivate::handleNewLocalConnections()
     create an HTTP server. Use bind() to start listening to all the incoming
     connections to a server.
 
-    This is a low level API, see \l QHttpServer for a highler level API to
+    This is a low level API, see \l QHttpServer for a higher level API to
     implement an HTTP server.
 */
 
@@ -154,13 +157,13 @@ QList<quint16> QAbstractHttpServer::serverPorts() const
 }
 
 /*!
-    Bind the HTTP server to given TCP \a server over which
-    the transmission happens. It is possible to call this function
+    Bind the given TCP \a server, over which the transmission happens,
+    to the HTTP server. It is possible to call this function
     multiple times with different instances of TCP \a server to
     handle multiple connections and ports, for example both SSL and
     non-encrypted connections.
 
-    After calling this function, every _new_ connection will be
+    After calling this function, every \e new connection will be
     handled and forwarded by the HTTP server.
 
     It is the user's responsibility to call QTcpServer::listen() on
@@ -195,12 +198,12 @@ bool QAbstractHttpServer::bind(QTcpServer *server)
 
 #if QT_CONFIG(localserver)
 /*!
-    Bind the HTTP server to given QLocalServer \a server over which
-    the transmission happens. It is possible to call this function
+    Bind the given QLocalServer \a server, over which the transmission
+    happens, to the HTTP server. It is possible to call this function
     multiple times with different instances of \a server to
     handle multiple connections.
 
-    After calling this function, every _new_ connection will be
+    After calling this function, every \e new connection will be
     handled and forwarded by the HTTP server.
 
     It is the user's responsibility to call QLocalServer::listen() on
@@ -233,7 +236,7 @@ bool QAbstractHttpServer::bind(QLocalServer *server)
 #endif
 
 /*!
-    Returns the TCP servers of this HTTP server.
+    Returns the TCP and SSL servers this HTTP server will handle connections from.
 
     \sa serverPorts()
  */
@@ -315,21 +318,22 @@ QAbstractHttpServer::verifyWebSocketUpgrade(const QHttpServerRequest &request) c
 }
 
 /*!
-    \fn template <typename Handler, QAbstractHttpServer::if_compatible_callable<Handler> = true> void QAbstractHttpServer::addWebSocketUpgradeVerifier(const typename QtPrivate::ContextTypeForFunctor<Handler>::ContextType *context, Handler &&func)
+    \fn template <typename Handler> void QAbstractHttpServer::addWebSocketUpgradeVerifier(const QObject *context, Handler &&func)
 
     Adds a callback function \a func that verifies incoming WebSocket
-    upgrades using the context object \a context. Upgrade attempts
-    succeed if at least one of the registered callback functions returns
-    \c Accept and a handler returning \c Deny has not been executed before
-    it. If no handlers are registered or all return \c PassToNext,
-    missingHandler() is called. The callback functions are executed in the
-    order they are registered. The callbacks cannot call
-    addWebSocketUpgradeVerifier().
+    upgrades. It is called using the provided \a context object.
+    An upgrade succeeds if at least one registered callback returns
+    \l{QHttpServerWebSocketUpgradeResponse::ResponseType::}{Accept} and
+    no prior callback has returned
+    \l{QHttpServerWebSocketUpgradeResponse::ResponseType::}{Deny}. If no
+    callbacks are registered, or all return
+    \l{QHttpServerWebSocketUpgradeResponse::ResponseType::}{PassToNext},
+    the missingHandler() function is called. Callbacks are executed in the
+    order they were registered, and they cannot call this function
+    themselves.
 
-    \note The WebSocket upgrades fail if no callbacks has been registered.
-    \note This overload participates in overload resolution only if the
-    callback function takes a \c const QHttpServerRequest & as an argument
-    and returns a QHttpServerWebSocketUpgradeResponse.
+    \note The \a func has to implement the signature
+    \c{QHttpServerWebSocketUpgradeResponse (*)(const QHttpServerRequest &)}.
 
     \code
     server.addWebSocketUpgradeVerifier(
@@ -371,8 +375,6 @@ void QAbstractHttpServer::addWebSocketUpgradeVerifierImpl(const QObject *context
     the \a request and sending the appropriate response back to \a responder.
     Return \c true if the \a request was handled successfully. If this method
     returns \c false, missingHandler() will be called afterwards.
-
-    This function must move out of \a responder before returning \c true.
 */
 
 /*!
@@ -420,6 +422,36 @@ void QAbstractHttpServer::setHttp2Configuration(const QHttp2Configuration &confi
 
 #endif
 
+/*!
+    \since 6.9
+
+    Sets this server's general configuration parameters to \a config.
+
+    \note The new configuration will be applied both to already established
+    connections and all next connections.
+
+    \sa configuration()
+*/
+void QAbstractHttpServer::setConfiguration(const QHttpServerConfiguration &config)
+{
+    Q_D(QAbstractHttpServer);
+    d->configuration = config;
+    d->requestFilter.setConfiguration(config);
+}
+
+/*!
+    \since 6.9
+
+    Returns this server's general configuration parameters.
+
+    \sa setConfiguration()
+*/
+QHttpServerConfiguration QAbstractHttpServer::configuration() const
+{
+    Q_D(const QAbstractHttpServer);
+    return d->configuration;
+}
+
 QT_END_NAMESPACE
 
 #include "moc_qabstracthttpserver.cpp"
