@@ -1,9 +1,11 @@
 // Copyright (C) 2022 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+// Qt-Security score:critical reason:network-protocol
 
 #include "qhttpserverrequest_p.h"
 
 #include <QtHttpServer/qhttpserverrequest.h>
+#include <QtNetwork/qhttpheaders.h>
 
 #include <QtCore/qdebug.h>
 #include <QtCore/qloggingcategory.h>
@@ -11,8 +13,9 @@
 #if QT_CONFIG(ssl)
 #include <QtNetwork/qsslsocket.h>
 #endif
-
-Q_LOGGING_CATEGORY(lc, "qt.httpserver.request")
+#if QT_CONFIG(http)
+#include <QtNetwork/private/qhttp2connection_p.h>
+#endif
 
 QT_BEGIN_NAMESPACE
 
@@ -32,17 +35,7 @@ Q_HTTPSERVER_EXPORT QDebug operator<<(QDebug debug, const QHttpServerRequest &re
     QDebugStateSaver saver(debug);
     debug.nospace() << "QHttpServerRequest(";
     debug << "(Url: " << request.url() << ")";
-    debug << "(Headers: ";
-    auto headers = request.headers();
-    bool firstHeader = true;
-    for (auto i = headers.begin(); i != headers.end(); ++i) {
-        if (firstHeader)
-            firstHeader = false;
-        else
-            debug << ", ";
-        debug << "(" << i->first << ": " << i->second << ")";
-    }
-    debug << ")";
+    debug << "(Headers: " << request.headers() << ")";
     debug << "(RemoteHost: " << request.remoteAddress() << ")";
     debug << "(BodySize: " << request.body().size() << ")";
     debug << ')';
@@ -51,6 +44,32 @@ Q_HTTPSERVER_EXPORT QDebug operator<<(QDebug debug, const QHttpServerRequest &re
 
 #endif
 
+namespace {
+
+QHttpServerRequest::Method parseRequestMethod(QByteArrayView str)
+{
+    if (str == "GET")
+        return QHttpServerRequest::Method::Get;
+    else if (str == "PUT")
+        return QHttpServerRequest::Method::Put;
+    else if (str == "DELETE")
+        return QHttpServerRequest::Method::Delete;
+    else if (str == "POST")
+        return QHttpServerRequest::Method::Post;
+    else if (str == "HEAD")
+        return QHttpServerRequest::Method::Head;
+    else if (str == "OPTIONS")
+        return QHttpServerRequest::Method::Options;
+    else if (str == "PATCH")
+        return QHttpServerRequest::Method::Patch;
+    else if (str == "CONNECT")
+        return QHttpServerRequest::Method::Connect;
+    else
+        return QHttpServerRequest::Method::Unknown;
+}
+
+} // anonymous namespace
+
 /*!
     \internal
 */
@@ -87,26 +106,10 @@ bool QHttpServerRequestPrivate::parseRequestLine(QByteArrayView line)
 
     parser.setMajorVersion(protocol[5] - '0');
     parser.setMinorVersion(protocol[7] - '0');
+    majorVersion = protocol[5] - '0';
+    minorVersion = protocol[7] - '0';
 
-    if (requestMethod == "GET")
-        method = QHttpServerRequest::Method::Get;
-    else if (requestMethod == "PUT")
-        method = QHttpServerRequest::Method::Put;
-    else if (requestMethod == "DELETE")
-        method = QHttpServerRequest::Method::Delete;
-    else if (requestMethod == "POST")
-        method = QHttpServerRequest::Method::Post;
-    else if (requestMethod == "HEAD")
-        method = QHttpServerRequest::Method::Head;
-    else if (requestMethod == "OPTIONS")
-        method = QHttpServerRequest::Method::Options;
-    else if (requestMethod == "PATCH")
-        method = QHttpServerRequest::Method::Patch;
-    else if (requestMethod == "CONNECT")
-        method = QHttpServerRequest::Method::Connect;
-    else
-        method = QHttpServerRequest::Method::Unknown;
-
+    method = parseRequestMethod(requestMethod);
     url = QUrl::fromEncoded(requestUrl.toByteArray());
     return true;
 }
@@ -114,7 +117,7 @@ bool QHttpServerRequestPrivate::parseRequestLine(QByteArrayView line)
 /*!
     \internal
 */
-qsizetype QHttpServerRequestPrivate::readRequestLine(QAbstractSocket *socket)
+qsizetype QHttpServerRequestPrivate::readRequestLine(QIODevice *socket)
 {
     if (fragment.isEmpty()) {
         // reserve bytes for the request line. This is better than always append() which reallocs
@@ -176,7 +179,7 @@ qint64 QHttpServerRequestPrivate::contentLength() const
 /*!
     \internal
 */
-qsizetype QHttpServerRequestPrivate::readHeader(QAbstractSocket *socket)
+qsizetype QHttpServerRequestPrivate::readHeader(QIODevice *socket)
 {
     if (fragment.isEmpty()) {
         // according to
@@ -252,7 +255,14 @@ qsizetype QHttpServerRequestPrivate::readHeader(QAbstractSocket *socket)
         QByteArray connectionHeaderField = headerField("connection");
         upgrade = connectionHeaderField.toLower().contains("upgrade");
 
-        state = (chunkedTransferEncoding || bodyLength > 0) ? State::ReadingData : State::AllDone;
+        if (chunkedTransferEncoding || bodyLength > 0) {
+            if (headerField("expect").compare("100-continue", Qt::CaseInsensitive) == 0)
+                state = State::ExpectContinue;
+            else
+                state = State::ReadingData;
+         } else {
+            state = State::AllDone;
+         }
     }
     return bytes;
 }
@@ -260,16 +270,51 @@ qsizetype QHttpServerRequestPrivate::readHeader(QAbstractSocket *socket)
 /*!
     \internal
 */
-QHttpServerRequestPrivate::QHttpServerRequestPrivate(const QHostAddress &remoteAddress)
-    : remoteAddress(remoteAddress)
+qsizetype QHttpServerRequestPrivate::sendContinue(QIODevice *socket)
 {
-    clear();
+    qsizetype ret = socket->write("HTTP/1.1 100 Continue\r\n\r\n");
+    state = State::ReadingData;
+    return ret;
 }
 
 /*!
     \internal
 */
-bool QHttpServerRequestPrivate::parse(QAbstractSocket *socket)
+QHttpServerRequestPrivate::QHttpServerRequestPrivate(const QHostAddress &remoteAddress,
+                                                     quint16 remotePort,
+                                                     const QHostAddress &localAddress,
+                                                     quint16 localPort)
+    : remoteAddress(remoteAddress),
+      remotePort(remotePort),
+      localAddress(localAddress),
+      localPort(localPort)
+{
+    clear();
+}
+
+#if QT_CONFIG(ssl)
+/*!
+    \internal
+*/
+QHttpServerRequestPrivate::QHttpServerRequestPrivate(const QHostAddress &remoteAddress,
+                                                     quint16 remotePort,
+                                                     const QHostAddress &localAddress,
+                                                     quint16 localPort,
+                                                     const QSslConfiguration &sslConfiguration)
+    : remoteAddress(remoteAddress),
+      remotePort(remotePort),
+      localAddress(localAddress),
+      localPort(localPort),
+      sslConfiguration(sslConfiguration)
+{
+    clear();
+}
+#endif
+
+/*!
+    \internal
+*/
+bool QHttpServerRequestPrivate::parse(QIODevice *socket)
 {
     qsizetype read;
 
@@ -287,6 +332,9 @@ bool QHttpServerRequestPrivate::parse(QAbstractSocket *socket)
         case State::ReadingHeader:
             read = readHeader(socket);
             continue;
+        case State::ExpectContinue:
+            read = sendContinue(socket);
+            continue;
         case State::ReadingData:
             if (chunkedTransferEncoding)
                 read = readRequestBodyChunked(socket);
@@ -306,6 +354,47 @@ bool QHttpServerRequestPrivate::parse(QAbstractSocket *socket)
     return read != -1;
 }
 
+#if QT_CONFIG(http)
+bool QHttpServerRequestPrivate::parse(QHttp2Stream *socket)
+{
+    parser.clear();
+
+    majorVersion = 2;
+    minorVersion = 0;
+
+    for (const auto &pair : socket->receivedHeaders()) {
+        if (pair.name == ":method") {
+            method = parseRequestMethod(pair.value);
+        } else if (pair.name == ":scheme") {
+            url.setScheme(QLatin1StringView(pair.value));
+        } else if (pair.name == ":authority") {
+            url.setAuthority(QLatin1StringView(pair.value));
+        } else if (pair.name == ":path") {
+            auto path = QUrl::fromEncoded(pair.value);
+            url.setPath(path.path());
+            url.setQuery(path.query());
+        } else {
+            parser.appendHeaderField(pair.name, pair.value);
+        }
+    }
+
+    if (url.scheme().isEmpty())
+        url.setScheme(u"https"_s);
+
+    if (url.host().isEmpty())
+        url.setHost(u"127.0.0.1"_s);
+
+    if (url.port() == -1)
+        url.setPort(port);
+
+    bodyLength = contentLength(); // cache the length
+
+    body = socket->downloadBuffer().readAll();
+
+    return true;
+}
+#endif
+
 /*!
     \internal
 */
@@ -319,6 +408,8 @@ void QHttpServerRequestPrivate::clear()
     currentChunkRead = 0;
     currentChunkSize = 0;
     upgrade = false;
+    majorVersion = 0;
+    minorVersion = 0;
 
     fragment.clear();
     bodyBuffer.clear();
@@ -332,7 +423,7 @@ void QHttpServerRequestPrivate::clear()
 */
 // note this function can only be used for non-chunked, non-compressed with
 // known content length
-qsizetype QHttpServerRequestPrivate::readBodyFast(QAbstractSocket *socket)
+qsizetype QHttpServerRequestPrivate::readBodyFast(QIODevice *socket)
 {
 
     qsizetype toBeRead = qMin(socket->bytesAvailable(), bodyLength - contentRead);
@@ -361,7 +452,7 @@ qsizetype QHttpServerRequestPrivate::readBodyFast(QAbstractSocket *socket)
 /*!
     \internal
 */
-qsizetype QHttpServerRequestPrivate::readRequestBodyRaw(QAbstractSocket *socket, qsizetype size)
+qsizetype QHttpServerRequestPrivate::readRequestBodyRaw(QIODevice *socket, qsizetype size)
 {
     // FIXME get rid of this function and just use readBodyFast and give it socket->bytesAvailable()
     qsizetype bytes = 0;
@@ -392,7 +483,7 @@ qsizetype QHttpServerRequestPrivate::readRequestBodyRaw(QAbstractSocket *socket,
 /*!
     \internal
 */
-qsizetype QHttpServerRequestPrivate::readRequestBodyChunked(QAbstractSocket *socket)
+qsizetype QHttpServerRequestPrivate::readRequestBodyChunked(QIODevice *socket)
 {
     qsizetype bytes = 0;
     while (socket->bytesAvailable()) {
@@ -454,7 +545,7 @@ qsizetype QHttpServerRequestPrivate::readRequestBodyChunked(QAbstractSocket *soc
 /*!
     \internal
 */
-qsizetype QHttpServerRequestPrivate::getChunkSize(QAbstractSocket *socket, qsizetype *chunkSize)
+qsizetype QHttpServerRequestPrivate::getChunkSize(QIODevice *socket, qsizetype *chunkSize)
 {
     qsizetype bytes = 0;
     char crlf[2];
@@ -534,10 +625,24 @@ qsizetype QHttpServerRequestPrivate::getChunkSize(QAbstractSocket *socket, qsize
 /*!
     \internal
 */
-QHttpServerRequest::QHttpServerRequest(const QHostAddress &remoteAddress) :
-    d(new QHttpServerRequestPrivate(remoteAddress))
+QHttpServerRequest::QHttpServerRequest(const QHostAddress &remoteAddress, quint16 remotePort,
+                                       const QHostAddress &localAddress, quint16 localPort)
+    : d(new QHttpServerRequestPrivate(remoteAddress, remotePort, localAddress, localPort))
 {}
 
+#if QT_CONFIG(ssl)
+/*!
+    \internal
+*/
+QHttpServerRequest::QHttpServerRequest(const QHostAddress &remoteAddress, quint16 remotePort,
+                                       const QHostAddress &localAddress, quint16 localPort,
+                                       const QSslConfiguration &sslConfiguration)
+    : d(new QHttpServerRequestPrivate(remoteAddress, remotePort,
+                                      localAddress, localPort,
+                                      sslConfiguration))
+{}
+#endif
+
 /*!
     Destroys a QHttpServerRequest
 */
@@ -577,13 +682,21 @@ QHttpServerRequest::Method QHttpServerRequest::method() const
 }
 
 /*!
+    \fn const QHttpHeaders &QHttpServerRequest::headers() const &
+    \fn QHttpHeaders QHttpServerRequest::headers() &&
+
     Returns all the request headers.
 */
-QList<QPair<QByteArray, QByteArray>> QHttpServerRequest::headers() const
+const QHttpHeaders &QHttpServerRequest::headers() const &
 {
     return d->parser.headers();
 }
 
+QHttpHeaders QHttpServerRequest::headers() &&
+{
+    return std::move(d->parser).headers();
+}
+
 /*!
     Returns the body of the request.
 */
@@ -600,6 +713,50 @@ QHostAddress QHttpServerRequest::remoteAddress() const
     return d->remoteAddress;
 }
 
+/*!
+    Returns the port of the origin host of the request.
+
+    \since 6.5
+*/
+quint16 QHttpServerRequest::remotePort() const
+{
+    return d->remotePort;
+}
+
+/*!
+    Returns the host address of the local socket which received the request.
+
+    \since 6.5
+*/
+QHostAddress QHttpServerRequest::localAddress() const
+{
+    return d->localAddress;
+}
+
+/*!
+    Returns the port of the local socket which received the request.
+
+    \since 6.5
+*/
+quint16 QHttpServerRequest::localPort() const
+{
+    return d->localPort;
+}
+
+#if QT_CONFIG(ssl)
+/*!
+    Returns the configuration of the established TLS connection.
+    The configurations will return true for isNull() if the connection
+    is not using TLS.
+
+    \since 6.7
+*/
+QSslConfiguration QHttpServerRequest::sslConfiguration() const
+{
+    return d->sslConfiguration;
+}
+#endif
+
 QT_END_NAMESPACE
 
 #include "moc_qhttpserverrequest.cpp"
