@@ -7,8 +7,6 @@
 #include <QtTest/qtest.h>
 #include <QtConcurrent/qtconcurrentrun.h>
 
-#include <QtCore/qoperatingsystemversion.h>
-#include <QtCore/qsystemdetection.h>
 #include <QtCore/qbytearray.h>
 #include <QtCore/qlist.h>
 #include <QtCore/qurl.h>
@@ -30,6 +28,8 @@
 #include <QtNetwork/qsslkey.h>
 #include <QtNetwork/qsslserver.h>
 
+#include <QtTest/private/qtesthelpers_p.h>
+
 constexpr char g_privateKey[] = R"(-----BEGIN RSA PRIVATE KEY-----
 MIIJKAIBAAKCAgEAvdrtZtVquwiG12+vd3OjRVibdK2Ob73DOOWgb5rIgQ+B2Uzc
 OFa0xsiRyc/bam9CEEqgn5YHSn95LJHvN3dbsA8vrFqIXTkisFAuHJqsmsYZbAIi
@@ -136,7 +136,7 @@ static int port = 0;
 #if QT_CONFIG(ssl)
 static int sslPort = 0;
 #endif
-static QString local = u"tst_qhttpservermultithreaded_local"_s;
+static const QString local = u"tst_qhttpservermultithreaded"_s + QString::number(QCoreApplication::applicationPid());
 // Simple HTTP 1.1 client
 class LocalHttpClient
 {
@@ -428,30 +428,10 @@ QString tst_QHttpServerMultithreaded::toString(qsizetype input) const
 
 void tst_QHttpServerMultithreaded::initTestCase_data()
 {
-    // Check if it's a macOS-build-with-SDK14 running on macOS 15:
-    auto sslServerIsBlockingKeychain = []
-    {
-    #ifdef Q_OS_MACOS
-    #if QT_MACOS_IOS_PLATFORM_SDK_EQUAL_OR_ABOVE(150000, 180000)
-        // Starting from macOS 15 our temporary keychain is ignored.
-        // We have to use kSecImportToMemoryOnly/kCFBooleanTrue key/value
-        // instead. This way we don't have to use QT_SSL_USE_TEMPORARY_KEYCHAIN anymore.
-        return false;
-    #else
-        if (QSslSocket::activeBackend() == QLatin1String("securetransport")
-            && QOperatingSystemVersion::current() >= QOperatingSystemVersion::MacOSSequoia) {
-            // We were built with SDK below 15, but a file-based keychains are not working anymore on macOS 15...
-            return true;
-        }
-    #endif // QT_MACOS_IOS_PLATFORM_SDK_EQUAL_OR_ABOVE
-    #endif // Q_OS_MACOS
-        return false;
-    };
-
     QTest::addColumn<ServerType>("serverType");
     QTest::addRow("TCP") << ServerType::TCP;
 #if QT_CONFIG(ssl)
-    if (QSslSocket::supportsSsl() && !sslServerIsBlockingKeychain())
+    if (QSslSocket::supportsSsl() && !QTestPrivate::isSecureTransportBlockingTest())
         QTest::addRow("SSL") << ServerType::SSL;
     else if (QSslSocket::supportsSsl())
         qInfo("Not testing TLS, keychain access will block the test");
@@ -493,9 +473,10 @@ void tst_QHttpServerMultithreaded::initTestCase()
 
     httpserver.route("/headers/", QHttpServerRequest::Method::Post,
                      [this](const QHttpServerRequest &request) {
-                         return QtConcurrent::run(&threadPool, [&request]() {
+                         auto headers = request.headers().toListOfPairs();
+                         return QtConcurrent::run(&threadPool, [headers]() {
                              QString result;
-                             for (auto header : request.headers().toListOfPairs())
+                             for (auto header : headers)
                                  result += QString::fromUtf8(header.first) + "\n";
                              return QHttpServerResponse(result);
                          });
